@@ -16,12 +16,17 @@ from enum import EnumMeta
 from keyword import iskeyword
 from numbers import Number
 from pathlib import Path
-from typing import Any, Dict, Iterator, List, Optional, Type, cast
+from typing import Any, Dict, Iterator, List, Optional, Tuple, Type, cast
 
 import minizinc
-from minizinc.error import parse_error
+from minizinc.error import MiniZincError, parse_error
 from minizinc.instance import Instance
-from minizinc.json import MZNJSONEncoder
+from minizinc.json import (
+    MZNJSONDecoder,
+    MZNJSONEncoder,
+    decode_async_json_stream,
+    decode_json_stream,
+)
 from minizinc.model import Method, Model, ParPath, UnknownExpression
 from minizinc.result import Result, Status, parse_solution, set_stat
 from minizinc.solver import Solver
@@ -47,6 +52,7 @@ class CLIInstance(Instance):
     _output: Optional[Dict[str, Type]] = None
     _method: Optional[Method] = None
     _parent: Optional["CLIInstance"] = None
+    _field_renames: List[Tuple[str, str]]
 
     def __init__(
         self,
@@ -56,6 +62,7 @@ class CLIInstance(Instance):
     ):
         super().__init__(solver, model)
         self._solver = solver
+        self._field_renames = []
         if driver is not None:
             self._driver = driver
         elif minizinc.default_driver is not None and isinstance(
@@ -181,9 +188,14 @@ class CLIInstance(Instance):
         with self.files() as files:
             assert len(files) > 0
             output = self._driver.run(["--model-interface-only"] + files, self._solver)
-        interface = json.loads(
-            output.stdout
-        )  # TODO: Possibly integrate with the MZNJSONDecoder
+        if self._driver.parsed_version >= (2, 6, 0):
+            interface = None
+            for obj in decode_json_stream(output.stdout):
+                if obj["type"] == "interface":
+                    interface = obj
+                    break
+        else:
+            interface = json.loads(output.stdout)
         old_method = self._method
         self._method = Method.from_string(interface["method"])
         self._input = {}
@@ -203,6 +215,7 @@ class CLIInstance(Instance):
             and (self._output != old_output or self._method != old_method)
         ):
             fields = []
+            self._field_renames = []
             if self._method is not Method.SATISFY and "objective" not in self._output:
                 fields.append(("objective", Number))
             for k, v in self._output.items():
@@ -214,6 +227,7 @@ class CLIInstance(Instance):
                         f"renamed to 'mzn_{k}'",
                         SyntaxWarning,
                     )
+                    self._field_renames.append((k, "mzn_" + k))
                     fields.append(("mzn_" + k, v))
                 else:
                     fields.append((k, v))
@@ -257,6 +271,7 @@ class CLIInstance(Instance):
         debug_output: Optional[Path] = None,
         **kwargs,
     ):
+        method = self.method  # Ensure self.analyse() has been executed
         # Set standard command line arguments
         cmd: List[Any] = [
             "--output-mode",
@@ -265,6 +280,9 @@ class CLIInstance(Instance):
             "--output-objective",
             "--output-output-item",
             "--statistics",  # Enable statistics
+            # Enable intermediate solutions
+            # (ensure that solvers always output their best solution)
+            "--intermediate-solutions",
         ]
 
         # Process number of solutions to be generated
@@ -274,7 +292,7 @@ class CLIInstance(Instance):
                     "The number of solutions cannot be limited when looking "
                     "for all solutions"
                 )
-            if self.method != Method.SATISFY:
+            if method != Method.SATISFY:
                 raise NotImplementedError(
                     "Finding all optimal solutions is not yet implemented"
                 )
@@ -294,22 +312,14 @@ class CLIInstance(Instance):
             if "-n" not in self._solver.stdFlags:
                 raise NotImplementedError("Solver does not support the -n flag")
             cmd.extend(["--num-solutions", str(nr_solutions)])
-        if "-i" in self._solver.stdFlags or "-a" in self._solver.stdFlags:
-            cmd.append("--intermediate-solutions")
         # Set number of processes to be used
         if processes is not None:
-            if "-p" not in self._solver.stdFlags:
-                raise NotImplementedError("Solver does not support the -p flag")
             cmd.extend(["--parallel", str(processes)])
         # Set random seed to be used
         if random_seed is not None:
-            if "-r" not in self._solver.stdFlags:
-                raise NotImplementedError("Solver does not support the -r flag")
             cmd.extend(["--random-seed", str(random_seed)])
         # Enable free search if specified
         if free_search:
-            if "-f" not in self._solver.stdFlags:
-                raise NotImplementedError("Solver does not support the -f flag")
             cmd.append("--free-search")
         # Set compiler optimisation level if specified
         if optimisation_level is not None:
@@ -331,31 +341,51 @@ class CLIInstance(Instance):
             else:
                 cmd.extend([flag, value])
 
-        self.method  # Ensure self.analyse() is executed
-
         # Add files as last arguments
         with self.files() as files, self._solver.configuration() as solver:
             assert self.output_type is not None
             cmd.extend(files)
-            # Run the MiniZinc process
-            proc = await self._driver.create_process(cmd, solver=solver)
-            assert isinstance(proc.stderr, asyncio.StreamReader)
-            assert isinstance(proc.stdout, asyncio.StreamReader)
-
-            # Python 3.7+: replace with asyncio.create_task
-            read_stderr = asyncio.ensure_future(_read_all(proc.stderr))
 
             status = Status.UNKNOWN
+            last_status = Status.UNKNOWN
             code = 0
+            statistics: Dict[str, Any] = {}
 
-            remainder: bytes = b""
             try:
-                async for raw_sol in _seperate_solutions(proc.stdout):
-                    status = Status.SATISFIED
-                    solution, statistics = parse_solution(
-                        raw_sol, self.output_type, self._enum_map
-                    )
-                    yield Result(Status.SATISFIED, solution, statistics)
+                # Run the MiniZinc process
+                proc = await self._driver.create_process(cmd, solver=solver)
+                assert isinstance(proc.stderr, asyncio.StreamReader)
+                assert isinstance(proc.stdout, asyncio.StreamReader)
+
+                # Python 3.7+: replace with asyncio.create_task
+                read_stderr = asyncio.ensure_future(_read_all(proc.stderr))
+
+                if self._driver.parsed_version >= (2, 6, 0):
+                    async for obj in decode_async_json_stream(
+                        proc.stdout, cls=MZNJSONDecoder, enum_map=self._enum_map
+                    ):
+                        solution, new_status, statistics = self._parse_stream_obj(
+                            obj, statistics
+                        )
+                        if new_status is not None:
+                            status = new_status
+                        elif solution is not None:
+                            if status == Status.UNKNOWN:
+                                status = Status.SATISFIED
+                            yield Result(status, solution, statistics)
+                            last_status = status
+                            solution = None
+                            statistics = {}
+                else:
+                    async for raw_sol in _seperate_solutions(proc.stdout):
+                        status = Status.SATISFIED
+                        solution, statistics = parse_solution(
+                            raw_sol,
+                            self.output_type,
+                            self._enum_map,
+                            self._field_renames,
+                        )
+                        yield Result(Status.SATISFIED, solution, statistics)
 
                 code = await proc.wait()
             except asyncio.IncompleteReadError as err:
@@ -363,33 +393,55 @@ class CLIInstance(Instance):
                 # Read remaining text in buffer
                 code = await proc.wait()
                 remainder = err.partial
-            except asyncio.CancelledError as e:
-                # Process was cancelled by the user.
-                # Terminate process and read remaining output
-                proc.terminate()
-                remainder = await _read_all(proc.stdout)
 
-                if isinstance(e, asyncio.CancelledError):
-                    raise
-            finally:
-                # parse the remaining statistics
-                for res in filter(None, remainder.split(SEPARATOR)):
-                    new_status = Status.from_output(res, self.method)
-                    if new_status is not None:
-                        status = new_status
-                    solution, statistics = parse_solution(
-                        res, self.output_type, self._enum_map
-                    )
-                    yield Result(status, solution, statistics)
-                # Raise error if required
-                stderr = None
-                if code != 0 or status == Status.ERROR:
-                    stderr = await read_stderr
-                    raise parse_error(stderr)
-                if debug_output is not None:
-                    if stderr is None:
-                        stderr = await read_stderr
-                    debug_output.write_bytes(stderr)
+                # Parse and output the remaining statistics and status messages
+                if self._driver.parsed_version >= (2, 6, 0):
+                    for obj in decode_json_stream(
+                        remainder, cls=MZNJSONDecoder, enum_map=self._enum_map
+                    ):
+                        solution, new_status, statistics = self._parse_stream_obj(
+                            obj, statistics
+                        )
+                        if new_status is not None:
+                            status = new_status
+                        elif solution is not None:
+                            if status == Status.UNKNOWN:
+                                status = Status.SATISFIED
+                            yield Result(status, solution, statistics)
+                            solution = None
+                            statistics = {}
+                else:
+                    for res in filter(None, remainder.split(SEPARATOR)):
+                        new_status = Status.from_output(res, method)
+                        if new_status is not None:
+                            status = new_status
+                        solution, statistics = parse_solution(
+                            res,
+                            self.output_type,
+                            self._enum_map,
+                            self._field_renames,
+                        )
+                        yield Result(status, solution, statistics)
+            except (asyncio.CancelledError, MiniZincError, Exception):
+                # Process was cancelled by the user, a MiniZincError occurred, or
+                # an unexpected Python exception occurred
+                # First, terminate the process
+                proc.terminate()
+                _ = await proc.wait()
+                # Then, reraise the error that occurred
+                raise
+            if self._driver.parsed_version >= (2, 6, 0) and (
+                status != last_status or statistics != {}
+            ):
+                yield Result(status, None, statistics)
+
+            # Raise error if required
+            stderr = await read_stderr
+            if code != 0 or status == Status.ERROR:
+                raise parse_error(stderr)
+
+            if debug_output is not None:
+                debug_output.write_bytes(stderr)
 
     @contextlib.contextmanager
     def flat(
@@ -473,6 +525,38 @@ class CLIInstance(Instance):
         self._reset_analysis()
         return super().add_string(code)
 
+    def _parse_stream_obj(self, obj, statistics):
+        solution = None
+        status = None
+        if obj["type"] == "solution":
+            tmp = obj["output"]["json"]
+            if "_objective" in tmp:
+                tmp["objective"] = tmp.pop("_objective")
+            if "_output" in tmp:
+                tmp["_output_item"] = tmp.pop("_output")
+            for before, after in self._field_renames:
+                tmp[after] = tmp.pop(before)
+
+            if "_checker" in statistics:
+                tmp["_checker"] = statistics.pop("_checker")
+
+            solution = self.output_type(**tmp)
+            statistics["time"] = obj["time"]
+        elif obj["type"] == "time":
+            statistics["time"] = obj["time"]
+        elif obj["type"] == "statistics":
+            statistics.update(obj["statistics"])
+        elif obj["type"] == "status":
+            status = Status.from_str(obj["status"])
+        elif obj["type"] == "checker":
+            if "raw" in obj["output"]:
+                statistics["_checker"] = obj["output"]["raw"]
+            else:
+                # TODO: can we ensure this is made JSON?
+                statistics["_checker"] = obj["output"]["dzn"]
+
+        return solution, status, statistics
+
 
 async def _seperate_solutions(stream: asyncio.StreamReader):
     solution: bytes = b""
