@@ -2,9 +2,12 @@
 #  License, v. 2.0. If a copy of the MPL was not distributed with this
 #  file, You can obtain one at http://mozilla.org/MPL/2.0/.
 
+import warnings
+
 from support import InstanceTestCase
 
-from minizinc import Method, Solver, Status
+from minizinc import Method, Solver, Status, default_driver
+from minizinc.error import MiniZincWarning
 from minizinc.helpers import check_result, check_solution
 
 
@@ -22,9 +25,14 @@ class CheckResults(InstanceTestCase):
 
     def test_incorrect(self):
         assert self.instance.method == Method.SATISFY
-        result = self.instance.solve()
-        result.solution = self.instance.output_type(x=[2, 1])
-        assert not check_result(self.instance, result, self.other_solver)
+        with warnings.catch_warnings(record=True) as w:
+            result = self.instance.solve()
+            result.solution = self.instance.output_type(x=[2, 1])
+            assert not check_result(self.instance, result, self.other_solver)
+            if default_driver.parsed_version >= (2, 6, 0):
+                assert len(w) == 1
+                assert issubclass(w[-1].category, MiniZincWarning)
+                assert "model inconsistency" in str(w[-1].message)
 
     def test_check_all(self):
         assert self.instance.method == Method.SATISFY
